@@ -1,3 +1,4 @@
+from getpass import getuser
 import re
 from sqlobject import col
 from sqlobject import dberrors
@@ -55,6 +56,9 @@ class PostgresConnection(DBAPI):
                 elif driver in ('py-postgresql', 'pypostgresql'):
                     from postgresql.driver import dbapi20
                     self.module = dbapi20
+                elif driver == 'pg8000':
+                    import pg8000
+                    self.module = pg8000
                 elif driver == 'pyodbc':
                     import pyodbc
                     self.module = pyodbc
@@ -71,7 +75,7 @@ class PostgresConnection(DBAPI):
                     raise ValueError(
                         'Unknown PostgreSQL driver "%s", '
                         'expected psycopg, psycopg2, psycopg1, '
-                        'pygresql, pypostgresql, '
+                        'pygresql, pypostgresql, pg8000, '
                         'odbc, pyodbc or pypyodbc' % driver)
             except ImportError:
                 pass
@@ -85,7 +89,7 @@ class PostgresConnection(DBAPI):
             # Register a converter for psycopg Binary type.
             registerConverter(type(self.module.Binary('')),
                               PsycoBinaryConverter)
-        elif driver in ('pygresql', 'py-postgresql', 'pypostgresql'):
+        elif driver in ('pygresql', 'py-postgresql', 'pypostgresql', 'pg8000'):
             registerConverter(type(self.module.Binary(b'')),
                               PostgresBinaryConverter)
         elif driver in ('odbc', 'pyodbc', 'pypyodbc'):
@@ -161,6 +165,12 @@ class PostgresConnection(DBAPI):
                 else:
                     if "unix" in dsn_dict:
                         del dsn_dict["unix"]
+            if driver == 'pg8000':
+                if host and host.startswith('/'):
+                    dsn_dict["host"] = None
+                    dsn_dict["unix_sock"] = host
+                if user is None:
+                    dsn_dict["user"] = getuser()
             self.dsn = dsn
         self.driver = driver
         self.unicodeCols = kw.pop('unicodeCols', False)
@@ -220,11 +230,16 @@ class PostgresConnection(DBAPI):
                     conn.setencoding(encoding=dbEncoding)
             self._executeRetry(conn, c,
                                "SET client_encoding TO '%s'" % dbEncoding)
+        c.close()
         return conn
 
     def _executeRetry(self, conn, cursor, query):
         if self.debug:
             self.printDebug(conn, query, 'QueryR')
+        dbEncoding = self.dbEncoding
+        if dbEncoding and isinstance(query, bytes) and (
+                self.driver == 'pg8000'):
+            query = query.decode(dbEncoding)
         try:
             return cursor.execute(query)
         except self.module.OperationalError as e:
@@ -242,7 +257,10 @@ class PostgresConnection(DBAPI):
             raise dberrors.InternalError(ErrorMessage(e))
         except self.module.ProgrammingError as e:
             msg = ErrorMessage(e)
-            if (len(e.args) >= 2) and e.args[1] == '23505':
+            if (
+                (len(e.args) > 2) and (e.args[1] == 'ERROR') and
+                    (e.args[2] == '23505')) \
+                    or ((len(e.args) >= 2) and (e.args[1] == '23505')):
                 raise dberrors.DuplicateEntryError(msg)
             else:
                 raise dberrors.ProgrammingError(msg)
@@ -286,6 +304,7 @@ class PostgresConnection(DBAPI):
         self._executeRetry(conn, c, q)
         if id is None:
             id = c.fetchone()[0]
+        c.close()
         if self.debugOutput:
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
